@@ -30,7 +30,6 @@ import logging
 import string
 import statvfs
 import time
-import gettext
 import datetime
 import threading
 import ConfigParser
@@ -38,10 +37,8 @@ from subprocess import Popen, PIPE
 
 from DistUpgradeGettext import gettext as _
 from DistUpgradeGettext import ngettext
-from DistUpgradeConfigParser import DistUpgradeConfig
-from DistUpgradeView import FuzzyTimeToStr
 
-from utils import inside_chroot
+from utils import inside_chroot, estimate_kernel_size_in_boot
 
 class CacheException(Exception):
     pass
@@ -51,7 +48,16 @@ class CacheExceptionDpkgInterrupted(CacheException):
     pass
 
 # the initrd/vmlinuz/abi space required in /boot for each kernel
-KERNEL_INITRD_SIZE = 19 * 1024 * 1024
+# we estimate based on the current kernel size and add a safety marging
+def _set_kernel_initrd_size():
+    size = estimate_kernel_size_in_boot()
+    if size == 0:
+        logging.warn("estimate_kernel_size_in_boot() returned '0'?")
+        size = 28*1024*1024
+    # add small safety buffer
+    size += 1*1024*1024
+    return size
+KERNEL_INITRD_SIZE = _set_kernel_initrd_size()
 
 class FreeSpaceRequired(object):
     """ FreeSpaceRequired object:
@@ -114,6 +120,15 @@ class MyCache(apt.Cache):
             not "RELEASE_UPGRADE_NO_RECOMMENDS" in os.environ):
             apt_pkg.Config.set("APT::Install-Recommends","true")
 
+    def _apply_dselect_upgrade(self):
+        """ honor the dselect install state """
+        for pkg in self:
+            if pkg.is_installed:
+                continue
+            if pkg._pkg.selected_state == apt_pkg.SELSTATE_INSTALL:
+                # upgrade() will take care of this
+                pkg.mark_install(auto_inst=False, auto_fix=False)
+
     @property
     def reqReinstallPkgs(self):
         " return the packages not downloadable packages in reqreinst state "
@@ -161,7 +176,7 @@ class MyCache(apt.Cache):
         apt_pkg.Config.set("Dir::Log",logdir)
         apt_pkg.Config.set("Dir::Log::Terminal","apt-term.log")
         self.logfd = os.open(os.path.join(logdir,"apt.log"),
-                             os.O_RDWR|os.O_CREAT|os.O_APPEND|os.O_SYNC, 0644)
+                             os.O_RDWR|os.O_CREAT|os.O_APPEND, 0644)
         os.write(self.logfd, "Log time: %s\n" % datetime.datetime.now())
         # turn on debugging in the cache
         apt_pkg.Config.set("Debug::pkgProblemResolver","true")
@@ -293,13 +308,16 @@ class MyCache(apt.Cache):
     def restore_snapshot(self):
         """ restore a snapshot """
         actiongroup = apt_pkg.ActionGroup(self._depcache)
+        # just make pyflakes shut up, later we need to use
+        # with self.actiongroup():
+        actiongroup
         self.clear()
         for name in self.to_remove:
             pkg = self[name]
             pkg.mark_delete()
         for name in self.to_install:
             pkg = self[name]
-            pkg.mark_install(autoFix=False, autoInst=False)
+            pkg.mark_install(auto_fix=False, auto_inst=False)
 
     def needServerMode(self):
         """ 
@@ -476,12 +494,11 @@ class MyCache(apt.Cache):
         try:
             # get new detection module and use the modalises files
             # from within the release-upgrader
-            nv = NvidiaDetection(datadir="./modaliases/",
-                                 obsolete="./nvidia-obsolete.pkgs")
+            nv = NvidiaDetection(obsolete="./nvidia-obsolete.pkgs")
             #nv = NvidiaDetection()
             # check if a binary driver is installed now
             for oldDriver in nv.oldPackages:
-                if self.has_key(oldDriver) and self[oldDriver].is_installed:
+                if oldDriver in self and self[oldDriver].is_installed:
                     self.mark_remove(oldDriver, "old nvidia driver")
                     break
             else:
@@ -490,7 +507,7 @@ class MyCache(apt.Cache):
             # check which one to use
             driver = nv.selectDriver()
             logging.debug("nv.selectDriver() returned '%s'" % driver)
-            if not self.has_key(driver):
+            if not driver in self:
                 logging.warning("no '%s' found" % driver)
                 return False
             if not (self[driver].marked_install or self[driver].marked_upgrade):
@@ -504,8 +521,12 @@ class MyCache(apt.Cache):
 
     def getKernelsFromBaseInstaller(self):
         """get the list of recommended kernels from base-installer"""
-        kernels = Popen(["/bin/sh", "./get_kernel_list.sh"],
-                        stdout=PIPE).communicate()[0]
+        p = Popen(["/bin/sh", "./get_kernel_list.sh"], stdout=PIPE)
+        res = p.wait()
+        if res != 0:
+            logging.warn("./get_kernel_list.sh returned non-zero exitcode")
+            return ""
+        kernels = p.communicate()[0]
         kernels = filter(lambda x : len(x) > 0,
                          map(string.strip, kernels.split("\n")))
         logging.debug("./get_kernel_list.sh returns: %s" % kernels)
@@ -518,7 +539,7 @@ class MyCache(apt.Cache):
         # check if we have a kernel from that list installed first
         kernels = self.getKernelsFromBaseInstaller()
         for kernel in kernels:
-            if not self.has_key(kernel):
+            if not kernel in self:
                 logging.debug("%s not available in cache" % kernel)
                 continue
             # this can happen e.g. on cdrom -> cdrom only upgrades
@@ -535,9 +556,25 @@ class MyCache(apt.Cache):
                 return 
         # if we have not found a kernel yet, use the first one that installs
         for kernel in kernels:
-            if self.mark_install(kernel, "Selecting new kernel from base-installer"):
+            if self.mark_install(kernel, 
+                                 "Selecting new kernel from base-installer"):
+                if self._has_kernel_headers_installed():
+                    prefix, sep, postfix = kernel.partition("-")
+                    headers = "%s-header-%s" % (prefix, postfix)
+                    self.mark_install(
+                        headers,
+                        "Selecting new kernel headers from base-installer")
+                else:
+                    logging.debug("no kernel-headers installed")
                 return
 
+    def _has_kernel_headers_installed(self):
+        for pkg in self:
+            if (pkg.name.startswith("linux-headers-") and
+                pkg.is_installed):
+                return True
+        return False
+
     def checkForKernel(self):
         """ check for the running kernel and try to ensure that we have
             an updated version
@@ -574,6 +611,8 @@ class MyCache(apt.Cache):
             if (pkg.candidateDownloadable and
                 not (pkg.is_installed or pkg.marked_install) and
                 not pkg.name in removeEssentialOk and
+                # ignore multiarch priority required packages
+                not ":" in pkg.name and
                 pkg.priority in need):
                 self.mark_install(pkg.name, "priority in required set '%s' but not scheduled for install" % need)
 
@@ -597,6 +636,9 @@ class MyCache(apt.Cache):
         t = threading.Thread(target=self.updateGUI, args=(self.view, lock,))
         t.start()
         try:
+            # mvo: disabled as it casues to many errornous installs
+            #self._apply_dselect_upgrade()
+
             # upgrade (and make sure this way that the cache is ok)
             self.upgrade(True)
 
@@ -640,17 +682,16 @@ class MyCache(apt.Cache):
                          " * Upgrading to a pre-release version of Ubuntu\n"
                          " * Running the current pre-release version of Ubuntu\n"
                          " * Unofficial software packages not provided by Ubuntu\n"
-                         "\n" % e)
+                         "\n") % e
             # we never have partialUpgrades (including removes) on a stable system
             # with only ubuntu sources so we do not recommend reporting a bug
             if partialUpgrade:
                 details += _("This is most likely a transient problem, "
                              "please try again later.")
             else:
-                details += _("If none of this applies, then please report this bug against "
-                             "the 'update-manager' package and include the files in "
-                             "/var/log/dist-upgrade/ in the bug report.")
-            # make the error text available again on stdout for the 
+                details += _("If none of this applies, then please report this bug using "
+                             "the command 'ubuntu-bug update-manager' in a terminal.")
+            # make the error text available again on stdout for the
             # text frontend
             self._stopAptResolverLog()
             view.error(_("Could not calculate the upgrade"), details)
@@ -678,7 +719,7 @@ class MyCache(apt.Cache):
                     if apt_pkg.version_compare(
                         ver.ver_str, pkg.installed.version) < 0:
                         for (verFileIter, index) in ver.file_list:
-                            indexfile = pkg._list.find_index(verFileIter)
+                            indexfile = pkg._pcache._list.find_index(verFileIter)
                             if indexfile and not indexfile.is_trusted:
                                 untrusted.append(pkg.name)
                                 break
@@ -728,7 +769,7 @@ class MyCache(apt.Cache):
                 raise SystemError, _("The package '%s' is marked for removal but it is in the removal blacklist.") % pkg.name
             if pkg.marked_delete and (pkg._pkg.Essential == True and
                                      not pkg.name in removeEssentialOk):
-                logging.debug("The package '%s' is marked for removal but it's a ESSENTIAL package", pkg.name)
+                logging.debug("The package '%s' is marked for removal but it's an ESSENTIAL package", pkg.name)
                 raise SystemError, _("The essential package '%s' is marked for removal.") % pkg.name
         # check bad-versions blacklist
         badVersions = self.config.getlist("Distro","BadVersions")
@@ -859,7 +900,9 @@ class MyCache(apt.Cache):
                         view.error(_("Can't install '%s'") % key,
                                    _("It was impossible to install a "
                                      "required package. Please report "
-                                     "this as a bug. "))
+                                     "this as a bug using "
+                                     "'ubuntu-bug update-manager' in "
+                                     "a terminal."))
                         return False
                     logging.debug("marked_install: '%s' -> '%s'" % (key, self[key].marked_install))
                     break
@@ -915,6 +958,9 @@ class MyCache(apt.Cache):
         # if it dosn't remove other packages depending on it
         # that are not obsolete as well
         actiongroup = apt_pkg.ActionGroup(self._depcache)
+        # just make pyflakes shut up, later we should use
+        # with self.actiongroup():
+        actiongroup
         self.create_snapshot()
         try:
             self[pkgname].markDelete(purge=purge)
@@ -1006,7 +1052,7 @@ class MyCache(apt.Cache):
                     foreign_pkgs.add(pkg.name)
         return foreign_pkgs
 
-    def checkFreeSpace(self):
+    def checkFreeSpace(self, snapshots_in_use=False):
         """
         this checks if we have enough free space on /var, /boot and /usr
         with the given cache 
@@ -1106,7 +1152,16 @@ class MyCache(apt.Cache):
             # the overlay dir
             for pkg in self:
                 if pkg.marked_upgrade or pkg.marked_install:
-                    required_for_aufs += self._depcache.get_candidate_ver(pkg._pkg).Size
+                    required_for_aufs += pkg.candidate.installed_size
+
+        # add old size of the package if we use snapshots
+        required_for_snapshots = 0.0
+        if snapshots_in_use:
+            for pkg in self:
+                if (pkg.is_installed and 
+                    (pkg.marked_upgrade or pkg.marked_delete)):
+                    required_for_snapshots += pkg.installed.installed_size
+            logging.debug("additional space for the snapshots: %s" % required_for_snapshots)
                     
         # sum up space requirements
         for (dir, size) in [(archivedir, self.requiredDownload),
@@ -1117,12 +1172,15 @@ class MyCache(apt.Cache):
                             ("/tmp", 5*1024*1024),   # /tmp for dkms LP: #427035
                             ("/", 10*1024*1024),     # small safety buffer /
                             (aufs_rw_dir, required_for_aufs),
+                            # if snapshots are in use
+                            ("/usr", required_for_snapshots),
                            ]:
             dir = os.path.realpath(dir)
             logging.debug("dir '%s' needs '%s' of '%s' (%f)" % (dir, size, fs_free[dir], fs_free[dir].free))
             fs_free[dir].free -= size
             fs_free[dir].need += size
 
+
         # check for space required violations
         required_list = {}
         for dir in fs_free:
@@ -1141,18 +1199,20 @@ class MyCache(apt.Cache):
 
 
 if __name__ == "__main__":
-	import DistUpgradeConfigParser
-        import DistUpgradeView
-        print "foo"
-	c = MyCache(DistUpgradeConfigParser.DistUpgradeConfig("."),
-                    DistUpgradeView.DistUpgradeView(), None)
-        #c.checkForNvidia()
-        #print c._identifyObsoleteKernels()
-        print c.checkFreeSpace()
-        sys.exit()
-	c.clear()
-        c.create_snapshot()
-        c.installedTasks
-        c.installTasks(["ubuntu-desktop"])
-        print c.get_changes()
-        c.restore_snapshot()
+    import sys
+    import DistUpgradeConfigParser
+    import DistUpgradeView
+    print "foo"
+    c = MyCache(DistUpgradeConfigParser.DistUpgradeConfig("."),
+                DistUpgradeView.DistUpgradeView(), None)
+    #c.checkForNvidia()
+    #print c._identifyObsoleteKernels()
+    print c.checkFreeSpace()
+    sys.exit()
+
+    c.clear()
+    c.create_snapshot()
+    c.installedTasks
+    c.installTasks(["ubuntu-desktop"])
+    print c.get_changes()
+    c.restore_snapshot()
