@@ -188,7 +188,7 @@ class GtkInstallProgressAdapter(InstallProgress):
         self.progress.set_text(" ")
         self.expander.set_sensitive(True)
         self.term.show()
-        # if no libgtk22-perl is installed show the terminal
+        # if no libgtk2-perl is installed show the terminal
         frontend= os.environ.get("DEBIAN_FRONTEND") or "gnome"
         if frontend == "gnome" and self._cache:
             if (not "libgtk2-perl" in self._cache or
@@ -410,7 +410,7 @@ class DistUpgradeViewGtk3(DistUpgradeView,SimpleGtkbuilderApp):
 
         icons = Gtk.IconTheme.get_default()
         try:
-            self.window_main.set_default_icon(icons.load_icon("update-manager", 32, 0))
+            self.window_main.set_default_icon(icons.load_icon("system-software-update", 32, 0))
         except GObject.GError, e:
             logging.debug("error setting default icon, ignoring (%s)" % e)
             pass
