@@ -20,7 +20,9 @@
 #  USA
 
 from gettext import gettext as _
-from stat import *
+from gettext import ngettext
+from stat import (S_IMODE, ST_MODE, S_IXUSR)
+from math import ceil
 
 import apt_pkg
 apt_pkg.init_config()
@@ -30,19 +32,22 @@ import logging
 import re
 import os
 import os.path
-import stat
+import glob
 import subprocess
 import sys
 import time
 import urllib2
 import urlparse
 
+from copy import copy
+from urlparse import urlsplit
+
 
 class ExecutionTime(object):
     """
     Helper that can be used in with statements to have a simple
-    measure of the timming of a particular block of code, e.g.
-    with ExecutinTime("db flush"):
+    measure of the timing of a particular block of code, e.g.
+    with ExecutionTime("db flush"):
         db.flush()
     """
     def __init__(self, info=""):
@@ -52,6 +57,23 @@ class ExecutionTime(object):
     def __exit__(self, type, value, stack):
         print "%s: %s" % (self.info, time.time() - self.now)
 
+def get_string_with_no_auth_from_source_entry(entry):
+    tmp = copy(entry)
+    url_parts = urlsplit(tmp.uri)
+    if url_parts.username:
+        tmp.uri = tmp.uri.replace(url_parts.username, "hidden-u")
+    if url_parts.password:
+        tmp.uri = tmp.uri.replace(url_parts.password, "hidden-p")
+    return str(tmp)
+
+def estimate_kernel_size_in_boot():
+    """ estimate the amount of space that the current kernel takes in /boot """
+    size = 0
+    kver = os.uname()[2]
+    for f in glob.glob("/boot/*%s*" % kver):
+        size += os.path.getsize(f)
+    return size
+
 def is_unity_running():
     """ return True if Unity is currently running """
     unity_running = False
@@ -214,7 +236,7 @@ def url_downloadable(uri, debug_func=None):
       return False
   return False
 
-def init_proxy(gconfclient=None):
+def init_proxy(gsettings=None):
   """ init proxy settings 
 
   * first check for http_proxy environment (always wins),
@@ -237,22 +259,22 @@ def init_proxy(gconfclient=None):
       proxy_port = str(cnf.find_i("Synaptic::httpProxyPort"))
       if proxy_host and proxy_port:
         proxy = "http://%s:%s/" % (proxy_host, proxy_port)
-  # then gconf
-  elif gconfclient:
-    try: # see LP: #281248
-      if gconfclient.get_bool("/system/http_proxy/use_http_proxy"):
-        host = gconfclient.get_string("/system/http_proxy/host")
-        port = gconfclient.get_int("/system/http_proxy/port")
-        use_auth = gconfclient.get_bool("/system/http_proxy/use_authentication")
-        if host and port:
-          if use_auth:
-            auth_user = gconfclient.get_string("/system/http_proxy/authentication_user")
-            auth_pw = gconfclient.get_string("/system/http_proxy/authentication_password")
-            proxy = "http://%s:%s@%s:%s/" % (auth_user,auth_pw,host, port)
-          else:
-            proxy = "http://%s:%s/" % (host, port)
-    except Exception, e:
-      print "error from gconf: %s" % e
+  # gconf is no more
+  # elif gconfclient:
+  #   try: # see LP: #281248
+  #     if gconfclient.get_bool("/system/http_proxy/use_http_proxy"):
+  #       host = gconfclient.get_string("/system/http_proxy/host")
+  #       port = gconfclient.get_int("/system/http_proxy/port")
+  #       use_auth = gconfclient.get_bool("/system/http_proxy/use_authentication")
+  #       if host and port:
+  #         if use_auth:
+  #           auth_user = gconfclient.get_string("/system/http_proxy/authentication_user")
+  #           auth_pw = gconfclient.get_string("/system/http_proxy/authentication_password")
+  #           proxy = "http://%s:%s@%s:%s/" % (auth_user,auth_pw,host, port)
+  #         else:
+  #           proxy = "http://%s:%s/" % (host, port)
+  #   except Exception, e:
+  #     print "error from gconf: %s" % e
   # if we have a proxy, set it
   if proxy:
     # basic verification
@@ -291,46 +313,22 @@ def on_battery():
     #print >>sys.stderr, "on_battery returned error: ", e
     return False
 
-def _inhibit_sleep_old_interface():
-  """
-  Send a dbus signal to org.gnome.PowerManager to not suspend
-  the system, this is to support upgrades from pre-gutsy g-p-m
-  """
-  import dbus
-  bus = dbus.Bus(dbus.Bus.TYPE_SESSION)
-  devobj = bus.get_object('org.gnome.PowerManager', 
-                          '/org/gnome/PowerManager')
-  dev = dbus.Interface(devobj, "org.gnome.PowerManager")
-  cookie = dev.Inhibit('UpdateManager', 'Updating system')
-  return (dev, cookie)
-
-def _inhibit_sleep_new_interface():
-  """
-  Send a dbus signal to gnome-power-manager to not suspend
-  the system
-  """
-  import dbus
-  bus = dbus.Bus(dbus.Bus.TYPE_SESSION)
-  devobj = bus.get_object('org.freedesktop.PowerManagement', 
-                          '/org/freedesktop/PowerManagement/Inhibit')
-  dev = dbus.Interface(devobj, "org.freedesktop.PowerManagement.Inhibit")
-  cookie = dev.Inhibit('UpdateManager', 'Updating system')
-  return (dev, cookie)
-
 def inhibit_sleep():
   """
   Send a dbus signal to power-manager to not suspend
-  the system, try both the new freedesktop and the
-  old gnome dbus interface
+  the system, using the freedesktop common interface
   """
   try:
-    return _inhibit_sleep_old_interface()
-  except Exception, e:
-    try:
-      return _inhibit_sleep_new_interface()
-    except Exception, e:
-      #print "could not send the dbus Inhibit signal: %s" % e
-      return (False, False)
+    import dbus
+    bus = dbus.Bus(dbus.Bus.TYPE_SESSION)
+    devobj = bus.get_object('org.freedesktop.PowerManagement', 
+                          '/org/freedesktop/PowerManagement/Inhibit')
+    dev = dbus.Interface(devobj, "org.freedesktop.PowerManagement.Inhibit")
+    cookie = dev.Inhibit('UpdateManager', 'Updating system')
+    return (dev, cookie)
+  except Exception:
+    #print "could not send the dbus Inhibit signal: %s" % e
+    return (False, False)
 
 def allow_sleep(dev, cookie):
   """Send a dbus signal to gnome-power-manager to allow a suspending
@@ -354,21 +352,38 @@ def get_lang():
     try:
         (locale_s, encoding) = locale.getdefaultlocale()
         return locale_s
-    except Exception, e: 
+    except Exception: 
         logging.exception("gedefaultlocale() failed")
         return None
 
+def get_ubuntu_flavor():
+    """ try to guess the flavor based on the running desktop """
+    # this will (of course) not work in a server environment, 
+    # but the main use case for this is to show the right
+    # release notes
+    denv = os.environ.get("DESKTOP_SESSION", "")
+    if "gnome" in denv:
+        return "ubuntu"
+    elif "kde" in denv:
+        return "kubuntu"
+    elif "xfce" in denv or "xubuntu" in denv:
+        return "xubuntu"
+    elif "LXDE" in denv or "Lubuntu" in denv:
+        return "lubuntu"
+    # default to ubuntu if nothing more specific is found
+    return "ubuntu"
+
 def error(parent, summary, message):
-  import gtk
-  d = gtk.MessageDialog(parent=parent,
-                        flags=gtk.DIALOG_MODAL,
-                        type=gtk.MESSAGE_ERROR,
-                        buttons=gtk.BUTTONS_CLOSE)
+  from gi.repository import Gtk, Gdk
+  d = Gtk.MessageDialog(parent=parent,
+                        flags=Gtk.DialogFlags.MODAL,
+                        type=Gtk.MessageType.ERROR,
+                        buttons=Gtk.ButtonsType.CLOSE)
   d.set_markup("<big><b>%s</b></big>\n\n%s" % (summary, message))
   d.realize()
-  d.window.set_functions(gtk.gdk.FUNC_MOVE)
+  d.window.set_functions(Gdk.FUNC_MOVE)
   d.set_title("")
-  res = d.run()
+  d.run()
   d.destroy()
   return False
 
@@ -376,23 +391,46 @@ def humanize_size(bytes):
     """
     Convert a given size in bytes to a nicer better readable unit
     """
-    if bytes == 0:
-        # TRANSLATORS: download size is 0
-        return _("0 KB")
-    elif bytes < 1024:
-        # TRANSLATORS: download size of very small updates
-        return _("1 KB")
-    elif bytes < 1024 * 1024:
-        # TRANSLATORS: download size of small updates, e.g. "250 KB"
-        return locale.format_string(_("%.0f KB"), bytes/1024)
+
+    if bytes < 1000 * 1000:
+        # to have 0 for 0 bytes, 1 for 0-1000 bytes and for 1 and above round up
+        size_in_kb = int(ceil(bytes/float(1000)));
+        # TRANSLATORS: download size of small updates, e.g. "250 kB"
+        return ngettext("%(size).0f kB", "%(size).0f kB", size_in_kb) % { "size" : size_in_kb }
     else:
         # TRANSLATORS: download size of updates, e.g. "2.3 MB"
-        return locale.format_string(_("%.1f MB"), bytes / 1024 / 1024)
+        return locale.format_string(_("%.1f MB"), bytes / 1000.0 / 1000.0)
 
 def get_arch():
     return apt_pkg.Config.find("APT::Architecture")
 
 
+def is_port_already_listening(port):
+    """ check if the current system is listening on the given tcp port """
+    # index in the line
+    INDEX_LOCAL_ADDR = 1
+    #INDEX_REMOTE_ADDR = 2
+    INDEX_STATE = 3
+    # state (st) that we care about
+    STATE_LISTENING = '0A'
+    # read the data
+    for line in open("/proc/net/tcp"):
+        line = line.strip()
+        if not line:
+            continue
+        # split, values are:
+        #   sl  local_address rem_address   st tx_queue rx_queue tr tm->when retrnsmt   uid  timeout inode                                                     
+        values = line.split()
+        state = values[INDEX_STATE]
+        if state != STATE_LISTENING:
+            continue
+        local_port_str = values[INDEX_LOCAL_ADDR].split(":")[1]
+        local_port = int(local_port_str, 16)
+        if local_port == port:
+            return True
+    return False
+
+
 def iptables_active():
     """ Return True if iptables is active """
     # FIXME: is there a better way?
@@ -407,6 +445,8 @@ target     prot opt source               destination
 """
     if os.getuid() != 0:
         raise OSError, "Need root to check the iptables state"
+    if not os.path.exists("/sbin/iptables"):
+        return False
     out = subprocess.Popen(["iptables", "-L"], 
                            stdout=subprocess.PIPE).communicate()[0]
     if out == iptables_empty:
