@@ -29,7 +29,6 @@
 from gi.repository import Gtk
 from gi.repository import Gdk
 from gi.repository import GObject
-from gi.repository import GLib
 from gi.repository import Gio
 GObject.threads_init()
 from gi.repository import Pango
@@ -54,7 +53,8 @@ import xml.sax.saxutils
 
 import dbus
 import dbus.service
-import dbus.glib
+from dbus.mainloop.glib import DBusGMainLoop
+DBusGMainLoop(set_as_default=True)
 
 import GtkProgress
 import backend
@@ -63,8 +63,7 @@ from gettext import gettext as _
 from gettext import ngettext
 
 
-from Core.utils import (is_unity_running,
-                        humanize_size, 
+from Core.utils import (humanize_size, 
                         init_proxy, 
                         on_battery,
                         inhibit_sleep,
@@ -105,7 +104,7 @@ def show_dist_no_longer_supported_dialog(parent=None):
         _("Your Ubuntu release is not supported anymore."),
         _("You will not get any further security fixes or critical "
           "updates. "
-          "Please upgrade to a later version of Ubuntu Linux."))
+          "Please upgrade to a later version of Ubuntu."))
     dialog = Gtk.MessageDialog(parent, 0, Gtk.MessageType.WARNING,
                                Gtk.ButtonsType.CLOSE,"")
     dialog.set_title("")
@@ -173,7 +172,7 @@ class UpdateManager(SimpleGtkbuilderApp):
 
   def __init__(self, datadir, options):
     self.setupDbus()
-    Gtk.Window.set_default_icon_name("update-manager")
+    Gtk.Window.set_default_icon_name("system-software-update")
     self.datadir = datadir
     SimpleGtkbuilderApp.__init__(self, datadir+"gtkbuilder/UpdateManager.ui",
                                  "update-manager")
@@ -188,7 +187,10 @@ class UpdateManager(SimpleGtkbuilderApp):
     self.sleep_cookie = None
     self.sleep_dev = None
 
-    self.image_logo.set_from_icon_name("update-manager", Gtk.IconSize.DIALOG)
+    # workaround for LP: #945536
+    self.clearing_store = False
+
+    self.image_logo.set_from_icon_name("system-software-update", Gtk.IconSize.DIALOG)
     self.window_main.set_sensitive(False)
     self.window_main.grab_focus()
     self.button_close.grab_focus()
@@ -224,9 +226,9 @@ class UpdateManager(SimpleGtkbuilderApp):
     cr.set_property("xpad", 6)
     cr.connect("toggled", self.toggled)
 
-    column_install = Gtk.TreeViewColumn("Install", cr, active=LIST_TOGGLE_CHECKED)
+    column_install = Gtk.TreeViewColumn(_("Install"), cr, active=LIST_TOGGLE_CHECKED)
     column_install.set_cell_data_func (cr, self.install_column_view_func)
-    column = Gtk.TreeViewColumn("Name", tr, markup=LIST_CONTENTS)
+    column = Gtk.TreeViewColumn(_("Name"), tr, markup=LIST_CONTENTS)
     column.set_resizable(True)
 
     column_install.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
@@ -438,7 +440,8 @@ class UpdateManager(SimpleGtkbuilderApp):
     # (even if currently disconnected)
     if self.cache.all_changes.has_key(name):
       changes = self.cache.all_changes[name]
-      self.set_changes_buffer(changes_buffer, changes[0], name, changes[1])
+      srcpkg = self.cache[name].sourcePackageName
+      self.set_changes_buffer(changes_buffer, changes, name, srcpkg)
     # if not connected, do not even attempt to get the changes
     elif not self.connected:
         changes_buffer.set_text(
@@ -509,12 +512,22 @@ class UpdateManager(SimpleGtkbuilderApp):
         menu.show()
         return True
 
+  # we need this for select all/unselect all
+  def _toggle_origin_headers(self, new_selection_value):
+    """ small helper that will set/unset the origin headers
+    """
+    model = self.treeview_update.get_model()
+    for row in model:
+        if not model.get_value(row.iter, LIST_PKG):
+            model.set_value(row.iter, LIST_TOGGLE_CHECKED, new_selection_value)
+
   def select_all_updgrades(self, widget):
     """
     Select all updates
     """
     self.setBusy(True)
     self.cache.saveDistUpgrade()
+    self._toggle_origin_headers(True)
     self.treeview_update.queue_draw()
     self.refresh_updates_count()
     self.setBusy(False)
@@ -525,6 +538,7 @@ class UpdateManager(SimpleGtkbuilderApp):
     """
     self.setBusy(True)
     self.cache.clear()
+    self._toggle_origin_headers(False)
     self.treeview_update.queue_draw()
     self.refresh_updates_count()
     self.setBusy(False)
@@ -565,12 +579,12 @@ class UpdateManager(SimpleGtkbuilderApp):
               self.unity.set_install_menuitem_visible(True)
           else:
               if inst_count > 0:
-                  download_str = ngettext("The update has already been downloaded, but not installed",
-                  "The updates have already been downloaded, but not installed", inst_count)
+                  download_str = ngettext("The update has already been downloaded, but not installed.",
+                  "The updates have already been downloaded, but not installed.", inst_count)
                   self.button_install.set_sensitive(True)
                   self.unity.set_install_menuitem_visible(True)
               else:
-                  download_str = _("There are no updates to install")
+                  download_str = _("There are no updates to install.")
                   self.button_install.set_sensitive(False)
                   self.unity.set_install_menuitem_visible(False)
               self.image_downsize.set_sensitive(False)
@@ -661,7 +675,7 @@ class UpdateManager(SimpleGtkbuilderApp):
       self.unity.set_updates_count(num_updates)
 
       if num_updates == 0:
-          text_header= "<big><b>%s</b></big>"  % _("Your system is up-to-date")
+          text_header= "<big><b>%s</b></big>"  % _("The software on this computer is up to date.")
           self.label_downsize.set_text("\n")
           if self.cache.keepCount() == 0:
               self.notebook_details.set_sensitive(False)
@@ -683,22 +697,14 @@ class UpdateManager(SimpleGtkbuilderApp):
           firstrun = self.settings.get_boolean("first-run")
           if firstrun:
               text_header = "<big><b>%s</b></big>" % _("Welcome to Ubuntu")
-              if is_unity_running():
-                  text_label_main = _("These software updates have been issued since this version of Ubuntu was released. If you don't want to install them now, choose \"Update Manager\" from Applications later.")
-              else:   
-                  text_label_main = _("These software updates have been issued since this version of Ubuntu was released. If you don't want to install them now, choose \"Update Manager\" from the Administration Menu later.")
+              text_label_main = _("These software updates have been issued since this version of Ubuntu was released.")
               self.settings.set_boolean("first-run", False)
           else:
-              text_header = "<big><b>%s</b></big>" % _("Software updates are available for this computer")
-              if is_unity_running():
-                  text_label_main = _("If you don't want to install them now, choose \"Update Manager\" from Applications later.")
-              else:
-                  text_label_main = _("If you don't want to install them now, choose \"Update Manager\" from the Administration menu later.")
+              text_header = "<big><b>%s</b></big>" % _("Software updates are available for this computer.")
           self.notebook_details.set_sensitive(True)
           self.treeview_update.set_sensitive(True)
           self.button_install.grab_default()
-          self.treeview_update.set_cursor(
-              Gtk.TreePath.new_from_string("1"), None, False)
+          self.treeview_update.set_cursor(Gtk.TreePath.new_from_string("1"), None, False)
       self.label_header.set_markup(text_header)
       self.label_main_details.set_text(text_label_main)
       return True
@@ -925,7 +931,6 @@ class UpdateManager(SimpleGtkbuilderApp):
     # make sure that we don't allow to toggle deactivated updates
     # this is needed for the call by the row activation callback
     if pkg is None:
-	
 	toggled_value = not self.store.get_value(iter, LIST_TOGGLE_CHECKED)
 	self.toggle_from_origin(pkg, origin, toggled_value)
 	self.store.set_value(iter, LIST_TOGGLE_CHECKED, toggled_value )
@@ -985,6 +990,10 @@ class UpdateManager(SimpleGtkbuilderApp):
   def fillstore(self):
     # use the watch cursor
     self.setBusy(True)
+    # disconnect the view first
+    self.treeview_update.set_model(None)
+    self.store.clear()
+    
     # clean most objects
     self.dl_size = 0
     try:
@@ -1006,8 +1015,8 @@ class UpdateManager(SimpleGtkbuilderApp):
         dialog.run()
         dialog.destroy()
         sys.exit(1)
-    self.store.clear()
     self.list = UpdateList(self)
+    
     while Gtk.events_pending():
         Gtk.main_iteration()
 
@@ -1068,6 +1077,8 @@ class UpdateManager(SimpleGtkbuilderApp):
       self.treeview_update.set_model(self.store)
     self.update_count()
     self.setBusy(False)
+    while Gtk.events_pending():
+      Gtk.main_iteration()
     self.check_all_updates_installable()
     self.refresh_updates_count()
     return False
